import java.awt.*;

class Elevator extends Panel
{
  static Color on = new Color(255, 255, 0);
  static Color off = new Color(255, 255, 255);
  static Color black = new Color(0, 0, 0);
  static Color white = new Color(255, 255, 255);

  private boolean opened, opened_changed;

  boolean stopped, upward;
  Simulation simulation;
  Label lights[];
  Label up, down;
  int pos;

  void reset()
  {
    pos = 0;
    stopped = upward = true;
    opened_changed = false;
    opened = false;
    setColors();
  }

  Elevator(Simulation s)
  {
    simulation = s;

    setBackground(white);
    setForeground(black);

    GridBagLayout l = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    setLayout(l);
    c.gridwidth = Simulation.LEVELS / 2;
    c.weighty = 1.0;
    c.insets = new Insets(4, 4, 4, 4);

    up = new Label("up");
    up.setAlignment(Label.CENTER);
    l.setConstraints(up, c);
    add(up);

    down = new Label("down");
    down.setAlignment(Label.CENTER);
    c.gridwidth = GridBagConstraints.REMAINDER;
    l.setConstraints(down, c);
    add(down);

    c.gridwidth = 1;
    c.weightx = 1.0;
    lights = new Label[Simulation.LEVELS];
    for(int i = 0; i < Simulation.LEVELS; i++)
      {
        lights[i] = new Label("" + i);
	lights[i].setAlignment(Label.CENTER);
	if(i == Simulation.LEVELS - 1)
	  c.gridwidth = GridBagConstraints.REMAINDER;
	l.setConstraints(lights[i], c);
	add(lights[i]);
      }

    reset();
  }
  
  void setColors()
  {
    for(int i = 0; i < Simulation.LEVELS; i++)
      {
        Color c = (i == pos ? on : off);

	if(c != lights[i].getBackground()) 
	  {
	    if(Simulation.DEBUG && (i == pos))
	      System.out.println("Elevator:                   pos    " + pos);

	    lights[i].setBackground(c);
	  }
      }

    if(stopped)
      {
	if(Simulation.DEBUG && (
	    up.getBackground() != off || down.getBackground() != off))
	  {
	    System.out.println("Elevator:                   stop");
	  }

	if(up.getBackground() != off) up.setBackground(off);
	if(down.getBackground() != off) down.setBackground(off);
      }
    else
      {
	if(upward) 
	  {
	    if(up.getBackground() != on)
	      {
		up.setBackground(on);
		if(Simulation.DEBUG)
		  System.out.println("Elevator:                   up");
	      }
	  }
	else
	  {
	    if(down.getBackground() != on)
	      {
		down.setBackground(on);
		if(Simulation.DEBUG)
		  System.out.println("Elevator:                   down");
	      }
	  }
      }

    if(Simulation.DEBUG && opened_changed)
      {
	System.out.print("Elevator:                   ");
	System.out.println(opened ? "opened" : "closed");
	opened_changed = false;
      }
  }

  void setOpened(boolean b)
  {
    opened_changed = (opened != b);
    opened = b;
  }

  boolean isOpened() { return opened; }

  synchronized void tick()
  {
    if(!stopped)
      {
        if(upward)
	  {
	    if(pos < Simulation.LEVELS - 1) pos++;
	    else stopped = true;
	  }
	else
	  {
	    if(pos > 0) pos--;
	    else stopped = true;
	  }
      }
    
    setColors();
  }
}
