import java.awt.*;
import java.awt.event.*;

class Level extends Panel implements ActionListener
{
  static Color pressed = new Color(255, 0, 0);
  static Color lightgrey = new Color(128, 128, 128);
  static Color white = new Color(255, 255, 255);
  static Color darkgrey = new Color(192, 192, 192);

  /* Reference to evelator allows access to 'elevator.pos' and 'elevator.opened'
   */
  Elevator elevator;

  /* The real state.
   */
  boolean requested;
  Button button;
  Door door;

  /* Part of the structure.
   */
  int pos;

  static Color unpressed(int pos)
  { 
    return pos % 2 == 0 ? darkgrey : lightgrey;
  }

  static Color background(int pos)
  { 
    return pos % 2 == 1 ? darkgrey : lightgrey;
  }

  Level(Elevator e, int i)
  {
    elevator = e;
    pos = i;
   
    setForeground(white);
    setBackground(background(pos));

    GridBagLayout l = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    setLayout(l);
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;

    c.gridwidth = 1;
    c.gridheight = 2;
    door = new Door(elevator, pos);
    l.setConstraints(door, c);
    add(door);

    Label label = new Label("Floor #" + pos);
    label.setAlignment(Label.CENTER);
    c.fill = GridBagConstraints.NONE;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.gridheight = 1;
    l.setConstraints(label, c);
    add(label);

    button = new Button("press");
    button.addActionListener(this);
    c.weighty = 2.0;
    c.weightx = 0.0;
    l.setConstraints(button, c);
    add(button);

    reset();
  }

  public void actionPerformed(ActionEvent e)
  {
    requested = true;
    setColors();
  }

  void setColors()
  {
    if(requested)
      {
	if(button.isEnabled())
	  {
	    if(Simulation.DEBUG) 
	      System.out.println("Level:                      req    " + pos);

	    button.setEnabled(false);
	    button.setBackground(pressed);
	  }
      }
    else
      {
	if(!button.isEnabled())
	  {
	    if(Simulation.DEBUG) 
	      System.out.println("Level:                      ack    " + pos);

	    button.setEnabled(true);
	    button.setBackground(unpressed(pos));
	  }
      }
  }

  synchronized void reset()
  {
    requested = false;
    door.reset();
    setColors();
  }

  synchronized void tick()
  {
    door.tick();
    setColors();
  }
}
