import java.awt.*;
import java.applet.*;
import java.awt.event.*;

public class Simulation 
  extends Applet 
  implements ActionListener, ItemListener
{
  static final int	LEVELS = 5;
  static final float	FREQUENCY = 1.5f;

  static final boolean	DEBUG = true;

  static final String	VERSION = "0.3"; 
  static final String	INFO = "Elevator Simulation " + VERSION;

  static final String	COPYRIGHT = 
    "(C)2000-2006 Armin Biere, ETH Zurich, JKU Linz";

  Button step_button, reset_button, stop_button, go_button, quit_button;
  Controller controller, controllers[];
  long num_ticks = 0;
  Elevator elevator;
  Level level[];
  Choice choice;
  Timer timer;
  
  Button addControlButton(Panel controls, String name)
  {
    Button button = new Button(name);
    controls.add(button);
    button.addActionListener(this);

    return button;
  }

  private void setup(boolean is_applet)
  {
    GridBagLayout l = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();

    setLayout(l);
    elevator = new Elevator(this);
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.BOTH;
    c.weighty = 0.0;
    c.weightx = 1.0;
    l.setConstraints(elevator, c);
    add(elevator);

    level = new Level[LEVELS];
    for(int i = LEVELS - 1; i >= 0; i--)
      {
	level[i] = new Level(elevator, i);
	c.weighty = 4.0;
	l.setConstraints(level[i], c);
	add(level[i]);
      }

    Panel controls = new Panel();
    controls.setBackground(new Color(255, 255, 255));
    controls.setLayout(new FlowLayout());
    reset_button = addControlButton(controls, "reset");
    if(!is_applet) quit_button = addControlButton(controls, "quit");
    stop_button = addControlButton(controls, "stop");
    stop_button.setEnabled(false);
    go_button = addControlButton(controls, "go");
    step_button = addControlButton(controls, "step");

    c.weighty = 0.0;
    l.setConstraints(controls, c);
    add(controls);

    controllers = new Controller[4];
    choice = new Choice();
    choice.add("SimpleController");
    controllers[0] = new SimpleController(this);
    choice.add("SafeController");
    controllers[1] = new SafeController(this);
    choice.add("LiveController");
    controllers[2] = new LiveController(this);
    choice.add("MinMaxController");
    controllers[3] = new MinMaxController(this);
    controller = controllers[0];

    Panel cpanel = new Panel();
    cpanel.add(choice);
    l.setConstraints(cpanel, c);
    choice.addItemListener(this);
    add(cpanel);

    Label info = new Label(INFO);
    info.setAlignment(Label.CENTER);
    l.setConstraints(info, c);
    add(info);

    Label copyright = new Label(COPYRIGHT);
    copyright.setAlignment(Label.CENTER);
    l.setConstraints(copyright, c);
    add(copyright);

    timer = new Timer((int)(1000 / FREQUENCY), this);
  }

  public Simulation() { setup(true); }
  public Simulation(boolean is_applet) { setup(is_applet); }

  synchronized void reset()
  {
    timer.stop();
    num_ticks = 0;
    elevator.reset();
    for(int i = 0; i < LEVELS; i++) level[i].reset();
    controller.reset();
    go_button.setEnabled(true);
    stop_button.setEnabled(false);
    step_button.setEnabled(true);
  }

  synchronized public void itemStateChanged(ItemEvent e)
  {
    controller = controllers[choice.getSelectedIndex()];
    reset();
  }

  public void actionPerformed(ActionEvent e)
  {
    Object src = e.getSource();
    if(src == quit_button) System.exit(0);
    else if(src == reset_button)
      {
	reset();
      }
    else if(src == stop_button)
      {
        timer.stop();
	stop_button.setEnabled(false);
	go_button.setEnabled(true);
	step_button.setEnabled(true);
      }
    else if(src == go_button)
      {
        timer.start();
	go_button.setEnabled(false);
	stop_button.setEnabled(true);
	step_button.setEnabled(false);
      }
    else tick();
  }

  synchronized void tick()
  {
    num_ticks++;
    controller.tick();
    elevator.tick();
    for(int i = 0; i < LEVELS; i++) level[i].tick();
    repaint();
    if(DEBUG)  System.out.println("Simulation.tick:            #" + num_ticks);
  }

  public static void main(String[] argv)
  {
    WindowListener l = new WindowAdapter() {
      public void windowClosing(WindowEvent e) { System.exit(0); }
    };

    Frame f = new Frame(INFO);
    f.addWindowListener(l);
    f.setSize(WIDTH, HEIGHT);
    f.add(new Simulation(false));
    f.setSize(400, 700);
    f.show();
  }

  public String getAppletInfo() { return INFO + " " + COPYRIGHT; }
}
